"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const useThrottledEffect_1 = __importDefault(require("../../../hooks/useThrottledEffect"));
const Button_1 = __importDefault(require("../Button"));
const reducer_1 = __importDefault(require("./reducer"));
const Condition_1 = __importDefault(require("./Condition"));
const field_types_1 = __importDefault(require("./field-types"));
const flattenTopLevelFields_1 = __importDefault(require("../../../../utilities/flattenTopLevelFields"));
require("./index.scss");
const baseClass = 'where-builder';
const validateWhereQuery = (query) => {
    if (query.or.length > 0 && query.or[0].and && query.or[0].and.length > 0) {
        return query;
    }
    return null;
};
const reduceFields = (fields) => flattenTopLevelFields_1.default(fields).reduce((reduced, field) => {
    if (typeof field_types_1.default[field.type] === 'object') {
        const formattedField = {
            label: field.label,
            value: field.name,
            ...field_types_1.default[field.type],
            props: {
                ...field,
            },
        };
        return [
            ...reduced,
            formattedField,
        ];
    }
    return reduced;
}, []);
const WhereBuilder = (props) => {
    const { collection, collection: { labels: { plural, } = {}, } = {}, handleChange, } = props;
    const [where, dispatchWhere] = react_1.useReducer(reducer_1.default, []);
    const [reducedFields] = react_1.useState(() => reduceFields(collection.fields));
    useThrottledEffect_1.default(() => {
        let whereQuery = {
            or: [],
        };
        if (where) {
            whereQuery.or = where.map((or) => or.reduce((conditions, condition) => {
                const { field, operator, value } = condition;
                if (field && operator && value) {
                    return {
                        and: [
                            ...conditions.and,
                            {
                                [field]: {
                                    [operator]: value,
                                },
                            },
                        ],
                    };
                }
                return conditions;
            }, {
                and: [],
            }));
        }
        whereQuery = validateWhereQuery(whereQuery);
        if (typeof handleChange === 'function')
            handleChange(whereQuery);
    }, 500, [where, handleChange]);
    return (react_1.default.createElement("div", { className: baseClass },
        where.length > 0 && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("div", { className: `${baseClass}__label` },
                "Filter",
                ' ',
                plural,
                ' ',
                "where"),
            react_1.default.createElement("ul", { className: `${baseClass}__or-filters` }, where.map((or, orIndex) => (react_1.default.createElement("li", { key: orIndex },
                orIndex !== 0 && (react_1.default.createElement("div", { className: `${baseClass}__label` }, "Or")),
                react_1.default.createElement("ul", { className: `${baseClass}__and-filters` }, or && or.map((_, andIndex) => (react_1.default.createElement("li", { key: andIndex },
                    andIndex !== 0 && (react_1.default.createElement("div", { className: `${baseClass}__label` }, "And")),
                    react_1.default.createElement(Condition_1.default, { value: where[orIndex][andIndex], orIndex: orIndex, andIndex: andIndex, key: andIndex, fields: reducedFields, dispatch: dispatchWhere }))))))))),
            react_1.default.createElement(Button_1.default, { className: `${baseClass}__add-or`, icon: "plus", buttonStyle: "icon-label", iconPosition: "left", iconStyle: "with-border", onClick: () => dispatchWhere({ type: 'add' }) }, "Or"))),
        where.length === 0 && (react_1.default.createElement("div", { className: `${baseClass}__no-filters` },
            react_1.default.createElement("div", { className: `${baseClass}__label` }, "No filters set"),
            react_1.default.createElement(Button_1.default, { className: `${baseClass}__add-first-filter`, icon: "plus", buttonStyle: "icon-label", iconPosition: "left", iconStyle: "with-border", onClick: () => dispatchWhere({ type: 'add' }) }, "Add filter")))));
};
exports.default = WhereBuilder;
