"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const object_to_formdata_1 = require("object-to-formdata");
const react_router_dom_1 = require("react-router-dom");
const react_toastify_1 = require("react-toastify");
const config_provider_1 = require("@payloadcms/config-provider");
const Locale_1 = require("../../utilities/Locale");
const api_1 = require("../../../api");
const useThrottledEffect_1 = __importDefault(require("../../../hooks/useThrottledEffect"));
const fieldReducer_1 = __importDefault(require("./fieldReducer"));
const initContextState_1 = __importDefault(require("./initContextState"));
const reduceFieldsToValues_1 = __importDefault(require("./reduceFieldsToValues"));
const getSiblingData_1 = __importDefault(require("./getSiblingData"));
const getDataByPath_1 = __importDefault(require("./getDataByPath"));
const wait_1 = __importDefault(require("../../../../utilities/wait"));
const buildInitialState_1 = __importDefault(require("./buildInitialState"));
const errorMessages_1 = __importDefault(require("./errorMessages"));
const context_1 = require("./context");
const baseClass = 'form';
const Form = (props) => {
    const { disabled, onSubmit, method, action, handleResponse, onSuccess, children, className, redirect, disableSuccessStatus, initialState, // fully formed initial field state
    initialData, // values only, paths are required as key - form should build initial state as convenience
    waitForAutocomplete, log, } = props;
    const history = react_router_dom_1.useHistory();
    const locale = Locale_1.useLocale();
    const { refreshCookie } = config_provider_1.useAuth();
    const [modified, setModified] = react_1.useState(false);
    const [processing, setProcessing] = react_1.useState(false);
    const [submitted, setSubmitted] = react_1.useState(false);
    const [formattedInitialData, setFormattedInitialData] = react_1.useState(buildInitialState_1.default(initialData));
    const contextRef = react_1.useRef({});
    let initialFieldState = {};
    if (formattedInitialData)
        initialFieldState = formattedInitialData;
    if (initialState)
        initialFieldState = initialState;
    // Allow access to initialState for field types such as Blocks and Array
    contextRef.current.initialState = initialState;
    const [fields, dispatchFields] = react_1.useReducer(fieldReducer_1.default, {}, () => initialFieldState);
    contextRef.current.fields = fields;
    const validateForm = react_1.useCallback(async () => {
        const validatedFieldState = {};
        let isValid = true;
        const validationPromises = Object.entries(contextRef.current.fields).map(async ([path, field]) => {
            const validatedField = { ...field };
            validatedField.valid = true;
            const validationResult = typeof field.validate === 'function' ? await field.validate(field.value) : true;
            if (typeof validationResult === 'string') {
                validatedField.errorMessage = validationResult;
                validatedField.valid = false;
                isValid = false;
            }
            validatedFieldState[path] = validatedField;
        });
        await Promise.all(validationPromises);
        dispatchFields({ type: 'REPLACE_STATE', state: validatedFieldState });
        return isValid;
    }, [contextRef]);
    const submit = react_1.useCallback(async (e) => {
        if (disabled) {
            e.preventDefault();
            return;
        }
        e.stopPropagation();
        e.preventDefault();
        setProcessing(true);
        if (waitForAutocomplete)
            await wait_1.default(100);
        const isValid = await contextRef.current.validateForm();
        setSubmitted(true);
        // If not valid, prevent submission
        if (!isValid) {
            react_toastify_1.toast.error('Please correct invalid fields.');
            setProcessing(false);
            return;
        }
        // If submit handler comes through via props, run that
        if (onSubmit) {
            onSubmit(fields, reduceFieldsToValues_1.default(fields));
            return;
        }
        const formData = contextRef.current.createFormData();
        try {
            const res = await api_1.requests[method.toLowerCase()](action, {
                body: formData,
            });
            setModified(false);
            if (typeof handleResponse === 'function') {
                handleResponse(res);
                return;
            }
            setProcessing(false);
            const contentType = res.headers.get('content-type');
            const isJSON = contentType && contentType.indexOf('application/json') !== -1;
            let json = {};
            if (isJSON)
                json = await res.json();
            if (res.status < 400) {
                setSubmitted(false);
                if (typeof onSuccess === 'function')
                    onSuccess(json);
                if (redirect) {
                    const destination = {
                        pathname: redirect,
                        state: {},
                    };
                    if (typeof json === 'object' && json.message && !disableSuccessStatus) {
                        destination.state = {
                            status: [
                                {
                                    message: json.message,
                                    type: 'success',
                                },
                            ],
                        };
                    }
                    history.push(destination);
                }
                else if (!disableSuccessStatus) {
                    react_toastify_1.toast.success(json.message || 'Submission successful.', { autoClose: 3000 });
                }
            }
            else {
                contextRef.current = { ...contextRef.current }; // triggers rerender of all components that subscribe to form
                if (json.message) {
                    react_toastify_1.toast.error(json.message);
                    return;
                }
                if (Array.isArray(json.errors)) {
                    const [fieldErrors, nonFieldErrors] = json.errors.reduce(([fieldErrs, nonFieldErrs], err) => {
                        const newFieldErrs = [];
                        const newNonFieldErrs = [];
                        if (err?.message) {
                            newNonFieldErrs.push(err);
                        }
                        if (Array.isArray(err?.data)) {
                            err.data.forEach((dataError) => {
                                if (dataError?.field) {
                                    newFieldErrs.push(dataError);
                                }
                                else {
                                    newNonFieldErrs.push(dataError);
                                }
                            });
                        }
                        return [
                            [
                                ...fieldErrs,
                                ...newFieldErrs,
                            ],
                            [
                                ...nonFieldErrs,
                                ...newNonFieldErrs,
                            ],
                        ];
                    }, [[], []]);
                    fieldErrors.forEach((err) => {
                        dispatchFields({
                            ...(contextRef.current?.fields?.[err.field] || {}),
                            valid: false,
                            errorMessage: err.message,
                            path: err.field,
                        });
                    });
                    nonFieldErrors.forEach((err) => {
                        react_toastify_1.toast.error(err.message || 'An unknown error occurred.');
                    });
                    return;
                }
                const message = errorMessages_1.default[res.status] || 'An unknown error occurrred.';
                react_toastify_1.toast.error(message);
            }
            return;
        }
        catch (err) {
            setProcessing(false);
            react_toastify_1.toast.error(err);
        }
    }, [
        action,
        disableSuccessStatus,
        disabled,
        fields,
        handleResponse,
        history,
        method,
        onSubmit,
        onSuccess,
        redirect,
        waitForAutocomplete,
    ]);
    const getFields = react_1.useCallback(() => contextRef.current.fields, [contextRef]);
    const getField = react_1.useCallback((path) => contextRef.current.fields[path], [contextRef]);
    const getData = react_1.useCallback(() => reduceFieldsToValues_1.default(contextRef.current.fields, true), [contextRef]);
    const getSiblingData = react_1.useCallback((path) => getSiblingData_1.default(contextRef.current.fields, path), [contextRef]);
    const getDataByPath = react_1.useCallback((path) => getDataByPath_1.default(contextRef.current.fields, path), [contextRef]);
    const getUnflattenedValues = react_1.useCallback(() => reduceFieldsToValues_1.default(contextRef.current.fields), [contextRef]);
    const createFormData = react_1.useCallback(() => {
        const data = reduceFieldsToValues_1.default(contextRef.current.fields);
        // nullAsUndefineds is important to allow uploads and relationship fields to clear themselves
        const formData = object_to_formdata_1.serialize(data, { indices: true, nullsAsUndefineds: false });
        return formData;
    }, [contextRef]);
    contextRef.current.dispatchFields = dispatchFields;
    contextRef.current.submit = submit;
    contextRef.current.getFields = getFields;
    contextRef.current.getField = getField;
    contextRef.current.getData = getData;
    contextRef.current.getSiblingData = getSiblingData;
    contextRef.current.getDataByPath = getDataByPath;
    contextRef.current.getUnflattenedValues = getUnflattenedValues;
    contextRef.current.validateForm = validateForm;
    contextRef.current.createFormData = createFormData;
    contextRef.current.setModified = setModified;
    contextRef.current.setProcessing = setProcessing;
    contextRef.current.setSubmitted = setSubmitted;
    contextRef.current.disabled = disabled;
    react_1.useEffect(() => {
        if (initialState) {
            contextRef.current = { ...initContextState_1.default };
            dispatchFields({ type: 'REPLACE_STATE', state: initialState });
        }
    }, [initialState]);
    react_1.useEffect(() => {
        if (initialData) {
            contextRef.current = { ...initContextState_1.default };
            const builtState = buildInitialState_1.default(initialData);
            setFormattedInitialData(builtState);
            dispatchFields({ type: 'REPLACE_STATE', state: builtState });
        }
    }, [initialData]);
    useThrottledEffect_1.default(() => {
        refreshCookie();
    }, 15000, [fields]);
    react_1.useEffect(() => {
        contextRef.current = { ...contextRef.current }; // triggers rerender of all components that subscribe to form
        setModified(false);
    }, [locale]);
    const classes = [
        className,
        baseClass,
    ].filter(Boolean).join(' ');
    if (log) {
        // eslint-disable-next-line no-console
        console.log(fields);
    }
    return (react_1.default.createElement("form", { noValidate: true, onSubmit: contextRef.current.submit, method: method, action: action, className: classes },
        react_1.default.createElement(context_1.FormContext.Provider, { value: contextRef.current },
            react_1.default.createElement(context_1.FormWatchContext.Provider, { value: {
                    fields,
                    ...contextRef.current,
                } },
                react_1.default.createElement(context_1.SubmittedContext.Provider, { value: submitted },
                    react_1.default.createElement(context_1.ProcessingContext.Provider, { value: processing },
                        react_1.default.createElement(context_1.ModifiedContext.Provider, { value: modified }, children)))))));
};
exports.default = Form;
