"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRenderedFields = void 0;
const react_1 = __importStar(require("react"));
const RenderCustomComponent_1 = __importDefault(require("../../utilities/RenderCustomComponent"));
const useIntersect_1 = __importDefault(require("../../../hooks/useIntersect"));
const baseClass = 'render-fields';
const intersectionObserverOptions = {
    rootMargin: '1000px',
};
const RenderedFieldContext = react_1.createContext({});
const useRenderedFields = () => react_1.useContext(RenderedFieldContext);
exports.useRenderedFields = useRenderedFields;
const RenderFields = (props) => {
    const { fieldSchema, fieldTypes, filter, permissions, readOnly: readOnlyOverride, operation: operationFromProps, className, } = props;
    const [hasRendered, setHasRendered] = react_1.useState(false);
    const [intersectionRef, entry] = useIntersect_1.default(intersectionObserverOptions);
    const isIntersecting = Boolean(entry?.isIntersecting);
    const isAboveViewport = entry?.boundingClientRect?.top < 0;
    const shouldRender = isIntersecting || isAboveViewport;
    const { operation: operationFromContext } = exports.useRenderedFields();
    const operation = operationFromProps || operationFromContext;
    const [contextValue, setContextValue] = react_1.useState({
        operation,
    });
    react_1.useEffect(() => {
        setContextValue({
            operation,
        });
    }, [operation]);
    react_1.useEffect(() => {
        if (shouldRender && !hasRendered) {
            setHasRendered(true);
        }
    }, [shouldRender, hasRendered]);
    const classes = [
        baseClass,
        className,
    ].filter(Boolean).join(' ');
    if (fieldSchema) {
        return (react_1.default.createElement("div", { ref: intersectionRef, className: classes }, hasRendered && (react_1.default.createElement(RenderedFieldContext.Provider, { value: contextValue }, fieldSchema.map((field, i) => {
            if (!field?.hidden && field?.admin?.disabled !== true) {
                if ((filter && typeof filter === 'function' && filter(field)) || !filter) {
                    const FieldComponent = field?.admin?.hidden ? fieldTypes.hidden : fieldTypes[field.type];
                    const fieldPermissions = field?.name ? permissions?.[field.name] : permissions;
                    let { admin: { readOnly } = {} } = field;
                    if (readOnlyOverride)
                        readOnly = true;
                    if (operation === 'create')
                        readOnly = false;
                    if (permissions?.[field?.name]?.read?.permission !== false) {
                        if (permissions?.[field?.name]?.[operation]?.permission === false) {
                            readOnly = true;
                        }
                        if (FieldComponent) {
                            return (react_1.default.createElement(RenderCustomComponent_1.default, { key: i, CustomComponent: field?.admin?.components?.Field, DefaultComponent: FieldComponent, componentProps: {
                                    ...field,
                                    path: field.path || field.name,
                                    fieldTypes,
                                    admin: {
                                        ...(field.admin || {}),
                                        readOnly,
                                    },
                                    permissions: fieldPermissions,
                                } }));
                        }
                        return (react_1.default.createElement("div", { className: "missing-field", key: i },
                            "No matched field found for",
                            ' ',
                            "\"",
                            field.label,
                            "\""));
                    }
                }
                return null;
            }
            return null;
        })))));
    }
    return null;
};
exports.default = RenderFields;
