"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_beautiful_dnd_1 = require("react-beautiful-dnd");
const withCondition_1 = __importDefault(require("../../withCondition"));
const Button_1 = __importDefault(require("../../../elements/Button"));
const DraggableSection_1 = __importDefault(require("../../DraggableSection"));
const rowReducer_1 = __importDefault(require("../rowReducer"));
const context_1 = require("../../Form/context");
const buildStateFromSchema_1 = __importDefault(require("../../Form/buildStateFromSchema"));
const useFieldType_1 = __importDefault(require("../../useFieldType"));
const Error_1 = __importDefault(require("../../Error"));
const validations_1 = require("../../../../../fields/validations");
const getDataByPath_1 = __importDefault(require("../../Form/getDataByPath"));
const Banner_1 = __importDefault(require("../../../elements/Banner"));
require("./index.scss");
const baseClass = 'field-type array';
const ArrayFieldType = (props) => {
    const { label, name, path: pathFromProps, fields, fieldTypes, validate = validations_1.array, required, maxRows, minRows, labels = {
        singular: 'Row',
        plural: 'Rows',
    }, permissions, admin: { readOnly, }, } = props;
    const [rows, dispatchRows] = react_1.useReducer(rowReducer_1.default, []);
    const { initialState, dispatchFields } = context_1.useForm();
    const path = pathFromProps || name;
    const memoizedValidate = react_1.useCallback((value) => {
        const validationResult = validate(value, { minRows, maxRows, required });
        return validationResult;
    }, [validate, maxRows, minRows, required]);
    const [disableFormData, setDisableFormData] = react_1.useState(false);
    const { showError, errorMessage, value, setValue, } = useFieldType_1.default({
        path,
        validate: memoizedValidate,
        disableFormData,
        ignoreWhileFlattening: true,
    });
    const addRow = react_1.useCallback(async (rowIndex) => {
        const subFieldState = await buildStateFromSchema_1.default(fields);
        dispatchFields({ type: 'ADD_ROW', rowIndex, subFieldState, path });
        dispatchRows({ type: 'ADD', rowIndex });
        setValue(value + 1);
    }, [dispatchRows, dispatchFields, fields, path, setValue, value]);
    const removeRow = react_1.useCallback((rowIndex) => {
        dispatchRows({ type: 'REMOVE', rowIndex });
        dispatchFields({ type: 'REMOVE_ROW', rowIndex, path });
    }, [dispatchRows, dispatchFields, path]);
    const moveRow = react_1.useCallback((moveFromIndex, moveToIndex) => {
        dispatchRows({ type: 'MOVE', moveFromIndex, moveToIndex });
        dispatchFields({ type: 'MOVE_ROW', moveFromIndex, moveToIndex, path });
    }, [dispatchRows, dispatchFields, path]);
    const onDragEnd = react_1.useCallback((result) => {
        if (!result.destination)
            return;
        const sourceIndex = result.source.index;
        const destinationIndex = result.destination.index;
        moveRow(sourceIndex, destinationIndex);
    }, [moveRow]);
    react_1.useEffect(() => {
        const data = getDataByPath_1.default(initialState, path);
        dispatchRows({ type: 'SET_ALL', data: data || [] });
    }, [initialState, path]);
    react_1.useEffect(() => {
        setValue(rows?.length || 0);
        if (rows?.length === 0) {
            setDisableFormData(false);
        }
        else {
            setDisableFormData(true);
        }
    }, [rows, setValue]);
    return (react_1.default.createElement(RenderArray, { onDragEnd: onDragEnd, label: label, showError: showError, errorMessage: errorMessage, rows: rows, labels: labels, addRow: addRow, removeRow: removeRow, moveRow: moveRow, path: path, name: name, fieldTypes: fieldTypes, fields: fields, permissions: permissions, value: value, readOnly: readOnly, minRows: minRows, maxRows: maxRows, required: required }));
};
const RenderArray = react_1.default.memo((props) => {
    const { onDragEnd, label, showError, errorMessage, rows, labels, addRow, removeRow, moveRow, path, fieldTypes, fields, permissions, value, readOnly, minRows, maxRows, required, } = props;
    const hasMaxRows = maxRows && rows.length >= maxRows;
    return (react_1.default.createElement(react_beautiful_dnd_1.DragDropContext, { onDragEnd: onDragEnd },
        react_1.default.createElement("div", { className: baseClass },
            react_1.default.createElement("header", { className: `${baseClass}__header` },
                react_1.default.createElement("h3", null, label),
                react_1.default.createElement(Error_1.default, { showError: showError, message: errorMessage })),
            react_1.default.createElement(react_beautiful_dnd_1.Droppable, { droppableId: "array-drop" }, (provided) => (react_1.default.createElement("div", Object.assign({ ref: provided.innerRef }, provided.droppableProps),
                rows.length > 0 && rows.map((row, i) => (react_1.default.createElement(DraggableSection_1.default, { readOnly: readOnly, key: row.key, id: row.key, blockType: "array", label: label, isOpen: row.open, rowCount: rows.length, rowIndex: i, addRow: addRow, removeRow: removeRow, moveRow: moveRow, parentPath: path, fieldTypes: fieldTypes, fieldSchema: fields, permissions: permissions, hasMaxRows: hasMaxRows }))),
                (rows.length < minRows || (required && rows.length === 0)) && (react_1.default.createElement(Banner_1.default, { type: "error" },
                    "This field requires at least",
                    ' ',
                    minRows
                        ? `${minRows} ${labels.plural}`
                        : `1 ${labels.singular}`)),
                (rows.length === 0 && readOnly) && (react_1.default.createElement(Banner_1.default, null,
                    "This field has no",
                    ' ',
                    labels.plural,
                    ".")),
                provided.placeholder))),
            (!readOnly && (!hasMaxRows)) && (react_1.default.createElement("div", { className: `${baseClass}__add-button-wrap` },
                react_1.default.createElement(Button_1.default, { onClick: () => addRow(value), buttonStyle: "icon-label", icon: "plus", iconStyle: "with-border", iconPosition: "left" }, `Add ${labels.singular}`))))));
});
exports.default = withCondition_1.default(ArrayFieldType);
