"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_beautiful_dnd_1 = require("react-beautiful-dnd");
const withCondition_1 = __importDefault(require("../../withCondition"));
const Button_1 = __importDefault(require("../../../elements/Button"));
const rowReducer_1 = __importDefault(require("../rowReducer"));
const context_1 = require("../../Form/context");
const buildStateFromSchema_1 = __importDefault(require("../../Form/buildStateFromSchema"));
const DraggableSection_1 = __importDefault(require("../../DraggableSection"));
const Error_1 = __importDefault(require("../../Error"));
const useFieldType_1 = __importDefault(require("../../useFieldType"));
const Popup_1 = __importDefault(require("../../../elements/Popup"));
const BlockSelector_1 = __importDefault(require("./BlockSelector"));
const validations_1 = require("../../../../../fields/validations");
const getDataByPath_1 = __importDefault(require("../../Form/getDataByPath"));
const Banner_1 = __importDefault(require("../../../elements/Banner"));
require("./index.scss");
const baseClass = 'field-type blocks';
const Blocks = (props) => {
    const { label, name, path: pathFromProps, blocks, labels = {
        singular: 'Block',
        plural: 'Blocks',
    }, fieldTypes, maxRows, minRows, required, validate = validations_1.blocks, permissions, admin: { readOnly, }, } = props;
    const path = pathFromProps || name;
    const [rows, dispatchRows] = react_1.useReducer(rowReducer_1.default, []);
    const { initialState, dispatchFields } = context_1.useForm();
    const memoizedValidate = react_1.useCallback((value) => {
        const validationResult = validate(value, {
            minRows, maxRows, labels, blocks, required,
        });
        return validationResult;
    }, [validate, maxRows, minRows, labels, blocks, required]);
    const [disableFormData, setDisableFormData] = react_1.useState(false);
    const { showError, errorMessage, value, setValue, } = useFieldType_1.default({
        path,
        validate: memoizedValidate,
        disableFormData,
        ignoreWhileFlattening: true,
    });
    const addRow = react_1.useCallback(async (rowIndex, blockType) => {
        const block = blocks.find((potentialBlock) => potentialBlock.slug === blockType);
        const subFieldState = await buildStateFromSchema_1.default(block.fields);
        dispatchFields({ type: 'ADD_ROW', rowIndex, subFieldState, path, blockType });
        dispatchRows({ type: 'ADD', rowIndex, blockType });
        setValue(value + 1);
    }, [path, setValue, value, blocks, dispatchFields]);
    const removeRow = react_1.useCallback((rowIndex) => {
        dispatchRows({ type: 'REMOVE', rowIndex });
        dispatchFields({ type: 'REMOVE_ROW', rowIndex, path });
        setValue(value - 1);
    }, [path, setValue, value, dispatchFields]);
    const moveRow = react_1.useCallback((moveFromIndex, moveToIndex) => {
        dispatchRows({ type: 'MOVE', moveFromIndex, moveToIndex });
        dispatchFields({ type: 'MOVE_ROW', moveFromIndex, moveToIndex, path });
    }, [dispatchRows, dispatchFields, path]);
    const toggleCollapse = react_1.useCallback((rowIndex) => {
        dispatchRows({ type: 'TOGGLE_COLLAPSE', rowIndex });
    }, []);
    const onDragEnd = react_1.useCallback((result) => {
        if (!result.destination)
            return;
        const sourceIndex = result.source.index;
        const destinationIndex = result.destination.index;
        moveRow(sourceIndex, destinationIndex);
    }, [moveRow]);
    react_1.useEffect(() => {
        const data = getDataByPath_1.default(initialState, path);
        dispatchRows({ type: 'SET_ALL', data: data || [] });
    }, [initialState, path]);
    react_1.useEffect(() => {
        setValue(rows?.length || 0);
        if (rows?.length === 0) {
            setDisableFormData(false);
        }
        else {
            setDisableFormData(true);
        }
    }, [rows, setValue]);
    return (react_1.default.createElement(RenderBlocks, { onDragEnd: onDragEnd, label: label, showError: showError, errorMessage: errorMessage, rows: rows, labels: labels, addRow: addRow, removeRow: removeRow, moveRow: moveRow, path: path, name: name, fieldTypes: fieldTypes, toggleCollapse: toggleCollapse, permissions: permissions, value: value, blocks: blocks, readOnly: readOnly, minRows: minRows, maxRows: maxRows, required: required }));
};
const RenderBlocks = react_1.default.memo((props) => {
    const { onDragEnd, label, showError, errorMessage, rows, labels, addRow, removeRow, moveRow, path, fieldTypes, permissions, value, toggleCollapse, blocks, readOnly, minRows, maxRows, required, } = props;
    const hasMaxRows = maxRows && rows.length >= maxRows;
    return (react_1.default.createElement(react_beautiful_dnd_1.DragDropContext, { onDragEnd: onDragEnd },
        react_1.default.createElement("div", { className: baseClass },
            react_1.default.createElement("header", { className: `${baseClass}__header` },
                react_1.default.createElement("h3", null, label),
                react_1.default.createElement(Error_1.default, { showError: showError, message: errorMessage })),
            react_1.default.createElement(react_beautiful_dnd_1.Droppable, { droppableId: "blocks-drop", isDropDisabled: readOnly }, (provided) => (react_1.default.createElement("div", Object.assign({ ref: provided.innerRef }, provided.droppableProps),
                rows.length > 0 && rows.map((row, i) => {
                    const { blockType } = row;
                    const blockToRender = blocks.find((block) => block.slug === blockType);
                    if (blockToRender) {
                        return (react_1.default.createElement(DraggableSection_1.default, { readOnly: readOnly, key: row.key, id: row.key, blockType: "blocks", blocks: blocks, label: blockToRender?.labels?.singular, isOpen: row.open, rowCount: rows.length, rowIndex: i, addRow: addRow, removeRow: removeRow, moveRow: moveRow, toggleRowCollapse: toggleCollapse, parentPath: path, fieldTypes: fieldTypes, permissions: permissions, hasMaxRows: hasMaxRows, fieldSchema: [
                                ...blockToRender.fields,
                                {
                                    name: 'blockType',
                                    type: 'text',
                                    admin: {
                                        hidden: true,
                                    },
                                },
                            ] }));
                    }
                    return null;
                }),
                (rows.length < minRows || (required && rows.length === 0)) && (react_1.default.createElement(Banner_1.default, { type: "error" },
                    "This field requires at least",
                    ' ',
                    minRows
                        ? `${minRows} ${labels.plural}`
                        : `1 ${labels.singular}`)),
                (rows.length === 0 && readOnly) && (react_1.default.createElement(Banner_1.default, null,
                    "This field has no",
                    ' ',
                    labels.plural,
                    ".")),
                provided.placeholder))),
            (!readOnly && (rows.length < maxRows || maxRows === undefined)) && (react_1.default.createElement("div", { className: `${baseClass}__add-button-wrap` },
                react_1.default.createElement(Popup_1.default, { buttonType: "custom", size: "large", horizontalAlign: "left", button: (react_1.default.createElement(Button_1.default, { buttonStyle: "icon-label", icon: "plus", iconPosition: "left", iconStyle: "with-border" }, `Add ${labels.singular}`)), render: ({ close }) => (react_1.default.createElement(BlockSelector_1.default, { blocks: blocks, addRow: addRow, addRowIndex: value, close: close })) }))))));
});
exports.default = withCondition_1.default(Blocks);
