"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const config_provider_1 = require("@payloadcms/config-provider");
const async_some_1 = __importDefault(require("async-some"));
const withCondition_1 = __importDefault(require("../../withCondition"));
const ReactSelect_1 = __importDefault(require("../../../elements/ReactSelect"));
const useFieldType_1 = __importDefault(require("../../useFieldType"));
const Label_1 = __importDefault(require("../../Label"));
const Error_1 = __importDefault(require("../../Error"));
const validations_1 = require("../../../../../fields/validations");
const context_1 = require("../../Form/context");
const optionsReducer_1 = __importDefault(require("./optionsReducer"));
require("./index.scss");
const maxResultsPerRequest = 10;
const baseClass = 'relationship';
const Relationship = (props) => {
    const { relationTo, validate = validations_1.relationship, path, name, required, label, hasMany, admin: { readOnly, style, width, } = {}, } = props;
    const { serverURL, routes: { api, }, collections, } = config_provider_1.useConfig();
    const formProcessing = context_1.useFormProcessing();
    const hasMultipleRelations = Array.isArray(relationTo);
    const [relations] = react_1.useState(Array.isArray(relationTo) ? relationTo : [relationTo]);
    const [options, dispatchOptions] = react_1.useReducer(optionsReducer_1.default, required ? [] : [{ value: 'null', label: 'None' }]);
    const [lastFullyLoadedRelation, setLastFullyLoadedRelation] = react_1.useState(-1);
    const [lastLoadedPage, setLastLoadedPage] = react_1.useState(1);
    const [search, setSearch] = react_1.useState('');
    const [errorLoading, setErrorLoading] = react_1.useState(false);
    const [hasLoadedFirstOptions, setHasLoadedFirstOptions] = react_1.useState(false);
    const memoizedValidate = react_1.useCallback((value) => {
        const validationResult = validate(value, { required });
        return validationResult;
    }, [validate, required]);
    const { value, showError, errorMessage, setValue, } = useFieldType_1.default({
        path: path || name,
        validate: memoizedValidate,
    });
    const addOptions = react_1.useCallback((data, relation) => {
        const collection = collections.find((coll) => coll.slug === relation);
        dispatchOptions({ type: 'ADD', data, relation, hasMultipleRelations, collection });
    }, [collections, hasMultipleRelations]);
    const getNextOptions = react_1.useCallback((params = {}) => {
        const clear = params?.clear;
        if (clear) {
            dispatchOptions({
                type: 'REPLACE',
                payload: required ? [] : [{ value: 'null', label: 'None' }],
            });
            setLastFullyLoadedRelation(-1);
        }
        if (!errorLoading) {
            const relationsToSearch = lastFullyLoadedRelation === -1 ? relations : relations.slice(lastFullyLoadedRelation + 1);
            if (relationsToSearch.length > 0) {
                async_some_1.default(relationsToSearch, async (relation, callback) => {
                    const collection = collections.find((coll) => coll.slug === relation);
                    const fieldToSearch = collection?.admin?.useAsTitle || 'id';
                    const searchParam = search ? `&where[${fieldToSearch}][like]=${search}` : '';
                    const response = await fetch(`${serverURL}${api}/${relation}?limit=${maxResultsPerRequest}&page=${lastLoadedPage}${searchParam}`);
                    const data = await response.json();
                    if (response.ok) {
                        if (data.hasNextPage) {
                            return callback(false, {
                                data,
                                relation,
                            });
                        }
                        return callback({ relation, data });
                    }
                    return setErrorLoading(true);
                }, (lastPage, nextPage) => {
                    if (nextPage) {
                        const { data, relation } = nextPage;
                        addOptions(data, relation);
                        setLastLoadedPage(lastLoadedPage + 1);
                    }
                    else {
                        const { data, relation } = lastPage;
                        addOptions(data, relation);
                        setLastFullyLoadedRelation(relations.indexOf(relation));
                        setLastLoadedPage(1);
                    }
                });
            }
        }
    }, [addOptions, api, collections, errorLoading, lastFullyLoadedRelation, lastLoadedPage, relations, required, search, serverURL]);
    const findOptionsByValue = react_1.useCallback(() => {
        if (value) {
            if (hasMany) {
                if (Array.isArray(value)) {
                    return value.map((val) => {
                        if (hasMultipleRelations) {
                            let matchedOption;
                            options.forEach((opt) => {
                                if (opt.options) {
                                    opt.options.some((subOpt) => {
                                        if (subOpt?.value === val.value) {
                                            matchedOption = subOpt;
                                            return true;
                                        }
                                        return false;
                                    });
                                }
                            });
                            return matchedOption;
                        }
                        return options.find((opt) => opt.value === val);
                    });
                }
                return undefined;
            }
            if (hasMultipleRelations) {
                let matchedOption;
                const valueWithRelation = value;
                options.forEach((opt) => {
                    if (opt?.options) {
                        opt.options.some((subOpt) => {
                            if (subOpt?.value === valueWithRelation.value) {
                                matchedOption = subOpt;
                                return true;
                            }
                            return false;
                        });
                    }
                });
                return matchedOption;
            }
            return options.find((opt) => opt.value === value);
        }
        return undefined;
    }, [hasMany, hasMultipleRelations, value, options]);
    const handleInputChange = react_1.useCallback((newSearch) => {
        if (search !== newSearch) {
            setSearch(newSearch);
            setLastFullyLoadedRelation(-1);
            setLastLoadedPage(1);
        }
    }, [search]);
    const addOptionByID = react_1.useCallback(async (id, relation) => {
        if (!errorLoading) {
            const response = await fetch(`${serverURL}${api}/${relation}/${id}?depth=0`);
            if (response.ok) {
                const data = await response.json();
                addOptions({ docs: [data] }, relation);
            }
            else {
                console.error(`There was a problem loading the document with ID of ${id}.`);
            }
        }
    }, [addOptions, api, errorLoading, serverURL]);
    react_1.useEffect(() => {
        if (value && hasLoadedFirstOptions) {
            if (hasMany) {
                const matchedOptions = findOptionsByValue();
                (matchedOptions || []).forEach((option, i) => {
                    if (!option) {
                        if (hasMultipleRelations) {
                            addOptionByID(value[i].value, value[i].relationTo);
                        }
                        else {
                            addOptionByID(value[i], relationTo);
                        }
                    }
                });
            }
            else {
                const matchedOption = findOptionsByValue();
                if (!matchedOption) {
                    if (hasMultipleRelations) {
                        const valueWithRelation = value;
                        addOptionByID(valueWithRelation.value, valueWithRelation.relationTo);
                    }
                    else {
                        addOptionByID(value, relationTo);
                    }
                }
            }
        }
    }, [addOptionByID, findOptionsByValue, hasMany, hasMultipleRelations, relationTo, value, hasLoadedFirstOptions]);
    react_1.useEffect(() => {
        const getFirstResults = async () => {
            const relation = relations[0];
            const res = await fetch(`${serverURL}${api}/${relation}?limit=${maxResultsPerRequest}&depth=0`);
            if (res.ok) {
                const data = await res.json();
                addOptions(data, relation);
                if (!data.hasNextPage) {
                    setLastFullyLoadedRelation(relations.indexOf(relation));
                }
                else {
                    setLastLoadedPage(2);
                }
                setHasLoadedFirstOptions(true);
            }
        };
        getFirstResults();
    }, [addOptions, api, relations, serverURL]);
    const classes = [
        'field-type',
        baseClass,
        showError && 'error',
        errorLoading && 'error-loading',
        readOnly && `${baseClass}--read-only`,
    ].filter(Boolean).join(' ');
    const valueToRender = (findOptionsByValue() || value);
    return (react_1.default.createElement("div", { className: classes, style: {
            ...style,
            width,
        } },
        react_1.default.createElement(Error_1.default, { showError: showError, message: errorMessage }),
        react_1.default.createElement(Label_1.default, { htmlFor: path, label: label, required: required }),
        !errorLoading && (react_1.default.createElement(ReactSelect_1.default, { isDisabled: readOnly, onInputChange: handleInputChange, onChange: !readOnly ? (selected) => {
                if (hasMany) {
                    setValue(selected ? selected.map((option) => {
                        if (hasMultipleRelations) {
                            return {
                                relationTo: option.relationTo,
                                value: option.value,
                            };
                        }
                        return option.value;
                    }) : null);
                }
                else {
                    setValue(selected.value);
                }
            } : undefined, onMenuScrollToBottom: getNextOptions, value: valueToRender, showError: showError, disabled: formProcessing, options: options, isMulti: hasMany })),
        errorLoading && (react_1.default.createElement("div", { className: `${baseClass}__error-loading` }, errorLoading))));
};
exports.default = withCondition_1.default(Relationship);
