"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const is_hotkey_1 = __importDefault(require("is-hotkey"));
const slate_react_1 = require("slate-react");
const slate_1 = require("slate");
const slate_history_1 = require("slate-history");
const validations_1 = require("../../../../../fields/validations");
const useFieldType_1 = __importDefault(require("../../useFieldType"));
const withCondition_1 = __importDefault(require("../../withCondition"));
const Label_1 = __importDefault(require("../../Label"));
const Error_1 = __importDefault(require("../../Error"));
const leaves_1 = __importDefault(require("./leaves"));
const elements_1 = __importDefault(require("./elements"));
const toggle_1 = __importDefault(require("./leaves/toggle"));
const hotkeys_1 = __importDefault(require("./hotkeys"));
const enablePlugins_1 = __importDefault(require("./enablePlugins"));
const defaultValue_1 = __importDefault(require("../../../../../fields/richText/defaultValue"));
const FieldTypeGutter_1 = __importDefault(require("../../FieldTypeGutter"));
const withHTML_1 = __importDefault(require("./plugins/withHTML"));
const mergeCustomFunctions_1 = __importDefault(require("./mergeCustomFunctions"));
require("./index.scss");
const defaultElements = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'ul', 'ol', 'link', 'relationship'];
const defaultLeaves = ['bold', 'italic', 'underline', 'strikethrough', 'code'];
const baseClass = 'rich-text';
const RichText = (props) => {
    const { path: pathFromProps, name, required, validate = validations_1.richText, label, admin, admin: { readOnly, style, width, placeholder, } = {}, } = props;
    const elements = admin?.elements || defaultElements;
    const leaves = admin?.leaves || defaultLeaves;
    const path = pathFromProps || name;
    const [loaded, setLoaded] = react_1.useState(false);
    const [enabledElements, setEnabledElements] = react_1.useState({});
    const [enabledLeaves, setEnabledLeaves] = react_1.useState({});
    const renderElement = react_1.useCallback(({ attributes, children, element }) => {
        const matchedElement = enabledElements[element?.type];
        const Element = matchedElement?.Element;
        if (Element) {
            return (react_1.default.createElement(Element, { attributes: attributes, element: element }, children));
        }
        return react_1.default.createElement("div", Object.assign({}, attributes), children);
    }, [enabledElements]);
    const renderLeaf = react_1.useCallback(({ attributes, children, leaf }) => {
        const matchedLeafName = Object.keys(enabledLeaves).find((leafName) => leaf[leafName]);
        if (enabledLeaves[matchedLeafName]?.Leaf) {
            const { Leaf } = enabledLeaves[matchedLeafName];
            return (react_1.default.createElement(Leaf, { attributes: attributes, leaf: leaf }, children));
        }
        return (react_1.default.createElement("span", Object.assign({}, attributes), children));
    }, [enabledLeaves]);
    const memoizedValidate = react_1.useCallback((value) => {
        const validationResult = validate(value, { required });
        return validationResult;
    }, [validate, required]);
    const fieldType = useFieldType_1.default({
        path,
        validate: memoizedValidate,
        stringify: true,
    });
    const { value, showError, setValue, errorMessage, } = fieldType;
    const classes = [
        baseClass,
        'field-type',
        showError && 'error',
        readOnly && `${baseClass}--read-only`,
    ].filter(Boolean).join(' ');
    const editor = react_1.useMemo(() => {
        let CreatedEditor = withHTML_1.default(slate_history_1.withHistory(slate_react_1.withReact(slate_1.createEditor())));
        CreatedEditor = enablePlugins_1.default(CreatedEditor, elements);
        CreatedEditor = enablePlugins_1.default(CreatedEditor, leaves);
        return CreatedEditor;
    }, [elements, leaves]);
    react_1.useEffect(() => {
        if (!loaded) {
            const mergedElements = mergeCustomFunctions_1.default(elements, elements_1.default);
            const mergedLeaves = mergeCustomFunctions_1.default(leaves, leaves_1.default);
            setEnabledElements(mergedElements);
            setEnabledLeaves(mergedLeaves);
            setLoaded(true);
        }
    }, [loaded, elements, leaves]);
    if (!loaded) {
        return null;
    }
    let valueToRender = value;
    if (typeof valueToRender === 'string')
        valueToRender = JSON.parse(valueToRender);
    if (!valueToRender)
        valueToRender = defaultValue_1.default;
    return (react_1.default.createElement("div", { className: classes, style: {
            ...style,
            width,
        } },
        react_1.default.createElement(FieldTypeGutter_1.default, null),
        react_1.default.createElement("div", { className: `${baseClass}__wrap` },
            react_1.default.createElement(Error_1.default, { showError: showError, message: errorMessage }),
            react_1.default.createElement(Label_1.default, { htmlFor: path, label: label, required: required }),
            react_1.default.createElement(slate_react_1.Slate, { editor: editor, value: valueToRender, onChange: (val) => {
                    if (val !== defaultValue_1.default && val !== value) {
                        setValue(val);
                    }
                } },
                react_1.default.createElement("div", { className: `${baseClass}__wrapper` },
                    react_1.default.createElement("div", { className: `${baseClass}__toolbar` },
                        elements.map((element, i) => {
                            let elementName;
                            if (typeof element === 'object' && element?.name)
                                elementName = element.name;
                            if (typeof element === 'string')
                                elementName = element;
                            const elementType = enabledElements[elementName];
                            const Button = elementType?.Button;
                            if (Button) {
                                return (react_1.default.createElement(Button, { key: i, path: path }));
                            }
                            return null;
                        }),
                        leaves.map((leaf, i) => {
                            let leafName;
                            if (typeof leaf === 'object' && leaf?.name)
                                leafName = leaf.name;
                            if (typeof leaf === 'string')
                                leafName = leaf;
                            const leafType = enabledLeaves[leafName];
                            const Button = leafType?.Button;
                            if (Button) {
                                return (react_1.default.createElement(Button, { key: i, path: path }));
                            }
                            return null;
                        })),
                    react_1.default.createElement(slate_react_1.Editable, { className: `${baseClass}__editor`, renderElement: renderElement, renderLeaf: renderLeaf, placeholder: placeholder, spellCheck: true, readOnly: readOnly, onKeyDown: (event) => {
                            Object.keys(hotkeys_1.default).forEach((hotkey) => {
                                if (is_hotkey_1.default(hotkey, event)) {
                                    event.preventDefault();
                                    const mark = hotkeys_1.default[hotkey];
                                    toggle_1.default(editor, mark);
                                }
                            });
                        } }))))));
};
exports.default = withCondition_1.default(RichText);
