"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withLinks = exports.wrapLink = exports.unwrapLink = exports.isLinkActive = void 0;
const slate_1 = require("slate");
const isLinkActive = (editor) => {
    const [link] = slate_1.Editor.nodes(editor, { match: (n) => n.type === 'link' });
    return !!link;
};
exports.isLinkActive = isLinkActive;
const unwrapLink = (editor) => {
    slate_1.Transforms.unwrapNodes(editor, { match: (n) => n.type === 'link' });
};
exports.unwrapLink = unwrapLink;
const wrapLink = (editor, url, newTab) => {
    if (exports.isLinkActive(editor)) {
        exports.unwrapLink(editor);
    }
    const { selection } = editor;
    const isCollapsed = selection && slate_1.Range.isCollapsed(selection);
    const link = {
        type: 'link',
        url,
        newTab,
        children: isCollapsed ? [{ text: url }] : [],
    };
    if (isCollapsed) {
        slate_1.Transforms.insertNodes(editor, link);
    }
    else {
        slate_1.Transforms.wrapNodes(editor, link, { split: true });
        slate_1.Transforms.collapse(editor, { edge: 'end' });
    }
};
exports.wrapLink = wrapLink;
const withLinks = (incomingEditor) => {
    const editor = incomingEditor;
    const { isInline } = editor;
    editor.isInline = (element) => {
        if (element.type === 'link') {
            return true;
        }
        return isInline(element);
    };
    return editor;
};
exports.withLinks = withLinks;
