"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_helmet_1 = __importDefault(require("react-helmet"));
const config_provider_1 = require("@payloadcms/config-provider");
const favicon_svg_1 = __importDefault(require("../../../assets/images/favicon.svg"));
const og_image_png_1 = __importDefault(require("../../../assets/images/og-image.png"));
const Meta = ({ description, lang = 'en', meta = [], title, keywords = 'CMS, Admin, Dashboard', }) => {
    const config = config_provider_1.useConfig();
    const titleSuffix = config.admin.meta?.titleSuffix ?? '- Payload';
    const favicon = config.admin.meta.favicon ?? favicon_svg_1.default;
    const ogImage = config.admin.meta.ogImage ?? og_image_png_1.default;
    return (react_1.default.createElement(react_helmet_1.default, { htmlAttributes: {
            lang,
        }, title: `${title} ${titleSuffix}`, meta: [
            {
                name: 'description',
                content: description,
            },
            {
                name: 'keywords',
                content: keywords,
            },
            {
                property: 'og:title',
                content: `${title} ${titleSuffix}`,
            },
            {
                property: 'og:image',
                content: ogImage,
            },
            {
                property: 'og:description',
                content: description,
            },
            {
                property: 'og:type',
                content: 'website',
            },
            {
                name: 'twitter:card',
                content: 'summary',
            },
            {
                name: 'twitter:title',
                content: title,
            },
            {
                name: 'twitter:description',
                content: description,
            },
        ].concat(meta), link: [
            {
                rel: 'icon',
                type: 'image/svg+xml',
                href: favicon,
            },
        ] }));
};
exports.default = Meta;
