"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_router_dom_1 = require("react-router-dom");
const config_provider_1 = require("@payloadcms/config-provider");
const Logo_1 = __importDefault(require("../../graphics/Logo"));
const Minimal_1 = __importDefault(require("../../templates/Minimal"));
const Form_1 = __importDefault(require("../../forms/Form"));
const Email_1 = __importDefault(require("../../forms/field-types/Email"));
const Password_1 = __importDefault(require("../../forms/field-types/Password"));
const Submit_1 = __importDefault(require("../../forms/Submit"));
const Button_1 = __importDefault(require("../../elements/Button"));
const Meta_1 = __importDefault(require("../../utilities/Meta"));
require("./index.scss");
const baseClass = 'login';
const Login = () => {
    const history = react_router_dom_1.useHistory();
    const { user, setToken } = config_provider_1.useAuth();
    const { admin: { user: userSlug }, serverURL, routes: { admin, api } } = config_provider_1.useConfig();
    const onSuccess = (data) => {
        if (data.token) {
            setToken(data.token);
            history.push(admin);
        }
    };
    if (user) {
        return (react_1.default.createElement(Minimal_1.default, { className: baseClass },
            react_1.default.createElement(Meta_1.default, { title: "Login", description: "Login user", keywords: "Login, Payload, CMS" }),
            react_1.default.createElement("div", { className: `${baseClass}__wrap` },
                react_1.default.createElement("h1", null, "Already logged in"),
                react_1.default.createElement("p", null,
                    "To log in with another user, you should",
                    ' ',
                    react_1.default.createElement(react_router_dom_1.Link, { to: `${admin}/logout` }, "log out"),
                    ' ',
                    "first."),
                react_1.default.createElement("br", null),
                react_1.default.createElement(Button_1.default, { el: "link", buttonStyle: "secondary", to: admin }, "Back to Dashboard"))));
    }
    return (react_1.default.createElement(Minimal_1.default, { className: baseClass },
        react_1.default.createElement(Meta_1.default, { title: "Login", description: "Login user", keywords: "Login, Payload, CMS" }),
        react_1.default.createElement("div", { className: `${baseClass}__brand` },
            react_1.default.createElement(Logo_1.default, null)),
        react_1.default.createElement(Form_1.default, { disableSuccessStatus: true, waitForAutocomplete: true, onSuccess: onSuccess, method: "post", action: `${serverURL}${api}/${userSlug}/login` },
            react_1.default.createElement(Email_1.default, { label: "Email Address", name: "email", admin: { autoComplete: 'email' }, required: true }),
            react_1.default.createElement(Password_1.default, { label: "Password", name: "password", autoComplete: "off", required: true }),
            react_1.default.createElement(react_router_dom_1.Link, { to: `${admin}/forgot` }, "Forgot password?"),
            react_1.default.createElement(Submit_1.default, null, "Login"))));
};
exports.default = Login;
