"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_router_dom_1 = require("react-router-dom");
const format_1 = __importDefault(require("date-fns/format"));
const config_provider_1 = require("@payloadcms/config-provider");
const Eyebrow_1 = __importDefault(require("../../../elements/Eyebrow"));
const Form_1 = __importDefault(require("../../../forms/Form"));
const Loading_1 = __importDefault(require("../../../elements/Loading"));
const PreviewButton_1 = __importDefault(require("../../../elements/PreviewButton"));
const Submit_1 = __importDefault(require("../../../forms/Submit"));
const RenderFields_1 = __importDefault(require("../../../forms/RenderFields"));
const CopyToClipboard_1 = __importDefault(require("../../../elements/CopyToClipboard"));
const DuplicateDocument_1 = __importDefault(require("../../../elements/DuplicateDocument"));
const DeleteDocument_1 = __importDefault(require("../../../elements/DeleteDocument"));
const Meta_1 = __importDefault(require("../../../utilities/Meta"));
const field_types_1 = __importDefault(require("../../../forms/field-types"));
const RenderTitle_1 = __importDefault(require("../../../elements/RenderTitle"));
const LeaveWithoutSaving_1 = __importDefault(require("../../../modals/LeaveWithoutSaving"));
const Auth_1 = __importDefault(require("./Auth"));
const Upload_1 = __importDefault(require("./Upload"));
require("./index.scss");
const baseClass = 'collection-edit';
const DefaultEditView = (props) => {
    const { params: { id } = {} } = react_router_dom_1.useRouteMatch();
    const { routes: { admin } } = config_provider_1.useConfig();
    const { collection, isEditing, data, onSave, permissions, isLoading, initialState, apiURL, action, hasSavePermission, } = props;
    const { slug, fields, admin: { useAsTitle, disableDuplicate, preview, }, timestamps, auth, upload, } = collection;
    const classes = [
        baseClass,
        isEditing && `${baseClass}--is-editing`,
    ].filter(Boolean).join(' ');
    return (react_1.default.createElement("div", { className: classes },
        react_1.default.createElement(Form_1.default, { className: `${baseClass}__form`, method: id ? 'put' : 'post', action: action, onSuccess: onSave, disabled: !hasSavePermission, initialState: initialState },
            react_1.default.createElement("div", { className: `${baseClass}__main` },
                react_1.default.createElement(Meta_1.default, { title: `${isEditing ? 'Editing' : 'Creating'} - ${collection.labels.singular}`, description: `${isEditing ? 'Editing' : 'Creating'} - ${collection.labels.singular}`, keywords: `${collection.labels.singular}, Payload, CMS` }),
                react_1.default.createElement(Eyebrow_1.default, null),
                react_1.default.createElement(LeaveWithoutSaving_1.default, null),
                react_1.default.createElement("div", { className: `${baseClass}__edit` },
                    isLoading && (react_1.default.createElement(Loading_1.default, null)),
                    !isLoading && (react_1.default.createElement(react_1.default.Fragment, null,
                        react_1.default.createElement("header", { className: `${baseClass}__header` },
                            react_1.default.createElement("h1", null,
                                react_1.default.createElement(RenderTitle_1.default, Object.assign({}, { data, useAsTitle, fallback: '[Untitled]' })))),
                        auth && (react_1.default.createElement(Auth_1.default, { useAPIKey: auth.useAPIKey, requirePassword: !isEditing, verify: auth.verify, collection: collection, email: data?.email })),
                        upload && (react_1.default.createElement(Upload_1.default, Object.assign({ data: data }, upload))),
                        react_1.default.createElement(RenderFields_1.default, { operation: isEditing ? 'update' : 'create', readOnly: !hasSavePermission, permissions: permissions.fields, filter: (field) => (!field?.admin?.position || (field?.admin?.position !== 'sidebar')), fieldTypes: field_types_1.default, fieldSchema: fields }))))),
            react_1.default.createElement("div", { className: `${baseClass}__sidebar` },
                react_1.default.createElement("div", { className: `${baseClass}__sidebar-sticky` },
                    react_1.default.createElement("div", { className: `${baseClass}__sidebar-sticky-wrap` },
                        isEditing ? (react_1.default.createElement("ul", { className: `${baseClass}__collection-actions` },
                            (permissions?.create?.permission) && (react_1.default.createElement(react_1.default.Fragment, null,
                                react_1.default.createElement("li", null,
                                    react_1.default.createElement(react_router_dom_1.Link, { to: `${admin}/collections/${slug}/create` }, "Create New")),
                                !disableDuplicate && (react_1.default.createElement("li", null,
                                    react_1.default.createElement(DuplicateDocument_1.default, { slug: slug }))))),
                            permissions?.delete?.permission && (react_1.default.createElement("li", null,
                                react_1.default.createElement(DeleteDocument_1.default, { collection: collection, id: id }))))) : undefined,
                        react_1.default.createElement("div", { className: `${baseClass}__document-actions${(preview && isEditing) ? ` ${baseClass}__document-actions--with-preview` : ''}` },
                            isEditing && (react_1.default.createElement(PreviewButton_1.default, { generatePreviewURL: preview })),
                            hasSavePermission && (react_1.default.createElement(Submit_1.default, null, "Save"))),
                        isEditing && (react_1.default.createElement("div", { className: `${baseClass}__api-url` },
                            react_1.default.createElement("span", { className: `${baseClass}__label` },
                                "API URL",
                                ' ',
                                react_1.default.createElement(CopyToClipboard_1.default, { value: apiURL })),
                            react_1.default.createElement("a", { href: apiURL, target: "_blank", rel: "noopener noreferrer" }, apiURL))),
                        !isLoading && (react_1.default.createElement(react_1.default.Fragment, null,
                            react_1.default.createElement("div", { className: `${baseClass}__sidebar-fields` },
                                react_1.default.createElement(RenderFields_1.default, { operation: isEditing ? 'update' : 'create', readOnly: !hasSavePermission, permissions: permissions.fields, filter: (field) => field?.admin?.position === 'sidebar', fieldTypes: field_types_1.default, fieldSchema: fields })),
                            isEditing && (react_1.default.createElement("ul", { className: `${baseClass}__meta` },
                                react_1.default.createElement("li", null,
                                    react_1.default.createElement("div", { className: `${baseClass}__label` }, "ID"),
                                    react_1.default.createElement("div", null, id)),
                                timestamps && (react_1.default.createElement(react_1.default.Fragment, null,
                                    data.updatedAt && (react_1.default.createElement("li", null,
                                        react_1.default.createElement("div", { className: `${baseClass}__label` }, "Last Modified"),
                                        react_1.default.createElement("div", null, format_1.default(new Date(data.updatedAt), 'MMMM do yyyy, h:mm a')))),
                                    data.createdAt && (react_1.default.createElement("li", null,
                                        react_1.default.createElement("div", { className: `${baseClass}__label` }, "Created"),
                                        react_1.default.createElement("div", null, format_1.default(new Date(data.createdAt), 'MMMM do yyyy, h:mm a'))))))))))))))));
};
exports.default = DefaultEditView;
