"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const useFieldType_1 = __importDefault(require("../../../../forms/useFieldType"));
const Button_1 = __importDefault(require("../../../../elements/Button"));
const FileDetails_1 = __importDefault(require("../../../../elements/FileDetails"));
const Error_1 = __importDefault(require("../../../../forms/Error"));
require("./index.scss");
const baseClass = 'file-field';
const handleDrag = (e) => {
    e.preventDefault();
    e.stopPropagation();
};
const validate = (value) => {
    if (!value && value !== undefined) {
        return 'A file is required.';
    }
    return true;
};
const File = (props) => {
    const inputRef = react_1.useRef(null);
    const dropRef = react_1.useRef(null);
    const [fileList, setFileList] = react_1.useState(undefined);
    const [selectingFile, setSelectingFile] = react_1.useState(false);
    const [dragging, setDragging] = react_1.useState(false);
    const [dragCounter, setDragCounter] = react_1.useState(0);
    const [replacingFile, setReplacingFile] = react_1.useState(false);
    const { data = {}, adminThumbnail, staticURL, } = props;
    const { filename } = data;
    const { value, setValue, showError, errorMessage, } = useFieldType_1.default({
        path: 'file',
        validate,
    });
    const handleDragIn = react_1.useCallback((e) => {
        e.preventDefault();
        e.stopPropagation();
        setDragCounter(dragCounter + 1);
        if (e.dataTransfer.items && e.dataTransfer.items.length > 0) {
            setDragging(true);
        }
    }, [dragCounter]);
    const handleDragOut = react_1.useCallback((e) => {
        e.preventDefault();
        e.stopPropagation();
        setDragCounter(dragCounter - 1);
        if (dragCounter > 1)
            return;
        setDragging(false);
    }, [dragCounter]);
    const handleDrop = react_1.useCallback((e) => {
        e.preventDefault();
        e.stopPropagation();
        setDragging(false);
        if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
            setFileList(e.dataTransfer.files);
            setDragging(false);
            e.dataTransfer.clearData();
            setDragCounter(0);
        }
        else {
            setDragging(false);
        }
    }, []);
    const handleInputChange = react_1.useCallback(() => {
        setSelectingFile(false);
        setFileList(inputRef?.current?.files || null);
        setValue(inputRef?.current?.files?.[0] || null);
    }, [inputRef, setValue]);
    react_1.useEffect(() => {
        if (selectingFile) {
            inputRef.current.click();
            setSelectingFile(false);
        }
    }, [selectingFile, inputRef, setSelectingFile]);
    react_1.useEffect(() => {
        const div = dropRef.current;
        if (div) {
            div.addEventListener('dragenter', handleDragIn);
            div.addEventListener('dragleave', handleDragOut);
            div.addEventListener('dragover', handleDrag);
            div.addEventListener('drop', handleDrop);
            return () => {
                div.removeEventListener('dragenter', handleDragIn);
                div.removeEventListener('dragleave', handleDragOut);
                div.removeEventListener('dragover', handleDrag);
                div.removeEventListener('drop', handleDrop);
            };
        }
        return () => null;
    }, [handleDragIn, handleDragOut, handleDrop, dropRef]);
    react_1.useEffect(() => {
        if (inputRef.current && fileList !== undefined) {
            inputRef.current.files = fileList;
        }
    }, [fileList]);
    react_1.useEffect(() => {
        setReplacingFile(false);
    }, [data]);
    const classes = [
        baseClass,
        dragging && `${baseClass}--dragging`,
        'field-type',
    ].filter(Boolean).join(' ');
    return (react_1.default.createElement("div", { className: classes },
        react_1.default.createElement(Error_1.default, { showError: showError, message: errorMessage }),
        (filename && !replacingFile) && (react_1.default.createElement(FileDetails_1.default, Object.assign({}, data, { staticURL: staticURL, adminThumbnail: adminThumbnail, handleRemove: () => {
                setReplacingFile(true);
                setFileList(null);
                setValue(null);
            } }))),
        (!filename || replacingFile) && (react_1.default.createElement("div", { className: `${baseClass}__upload` },
            value && (react_1.default.createElement("div", { className: `${baseClass}__file-selected` },
                react_1.default.createElement("span", { className: `${baseClass}__filename` }, value.name),
                react_1.default.createElement(Button_1.default, { icon: "x", round: true, buttonStyle: "icon-label", iconStyle: "with-border", onClick: () => {
                        setFileList(null);
                        setValue(null);
                    } }))),
            !value && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("div", { className: `${baseClass}__drop-zone`, ref: dropRef },
                    react_1.default.createElement(Button_1.default, { size: "small", buttonStyle: "secondary", onClick: () => setSelectingFile(true) }, "Select a file"),
                    react_1.default.createElement("span", { className: `${baseClass}__drag-label` }, "or drag and drop a file here")))),
            react_1.default.createElement("input", { ref: inputRef, type: "file", onChange: handleInputChange })))));
};
exports.default = File;
