"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable react/jsx-max-props-per-line */
const react_1 = __importDefault(require("react"));
const react_2 = require("@testing-library/react");
const Blocks_1 = __importDefault(require("./field-types/Blocks"));
const Date_1 = __importDefault(require("./field-types/Date"));
const Checkbox_1 = __importDefault(require("./field-types/Checkbox"));
describe('Cell Types', () => {
    describe('Blocks', () => {
        const field = {
            label: 'Blocks Content',
            name: 'blocks',
            labels: {
                singular: 'Block',
            },
            type: 'blocks',
            blocks: [
                {
                    slug: 'number',
                    labels: {
                        singular: 'Number',
                    },
                },
            ],
        };
        it('renders multiple', () => {
            const data = [
                { blockType: 'number' },
                { blockType: 'number' },
            ];
            const { container } = react_2.render(react_1.default.createElement(Blocks_1.default, { data: data, field: field }));
            const el = container.querySelector('span');
            expect(el).toHaveTextContent('2 Blocks Content - Number, Number');
        });
        it('renders zero', () => {
            const data = [];
            const { container } = react_2.render(react_1.default.createElement(Blocks_1.default, { data: data, field: field }));
            const el = container.querySelector('span');
            expect(el).toHaveTextContent('0 Blocks Content');
        });
        it('renders "and X more" if over maximum of 5', () => {
            const data = [
                { blockType: 'number' },
                { blockType: 'number' },
                { blockType: 'number' },
                { blockType: 'number' },
                { blockType: 'number' },
                { blockType: 'number' },
            ];
            const { container } = react_2.render(react_1.default.createElement(Blocks_1.default, { data: data, field: field }));
            const el = container.querySelector('span');
            expect(el).toHaveTextContent('6 Blocks Content - Number, Number, Number, Number, Number and 1 more');
        });
    });
    describe('Date', () => {
        it('renders date', () => {
            const timeStamp = '2020-10-06T14:07:39.033Z';
            const { container } = react_2.render(react_1.default.createElement(Date_1.default, { data: timeStamp }));
            const dateMatch = /October\s6th\s2020,\s[\d]{1,2}:07\s[A|P]M/; // Had to account for timezones in CI
            const el = container.querySelector('span');
            expect(el.textContent).toMatch(dateMatch);
        });
        it('handles undefined', () => {
            const timeStamp = undefined;
            const { container } = react_2.render(react_1.default.createElement(Date_1.default, { data: timeStamp }));
            const el = container.querySelector('span');
            expect(el.textContent).toBe('');
        });
    });
    describe('Checkbox', () => {
        it('renders true', () => {
            const { container } = react_2.render(react_1.default.createElement(Checkbox_1.default, { data: true }));
            const el = container.querySelector('span');
            expect(el).toHaveTextContent('true');
        });
        it('renders false', () => {
            const { container } = react_2.render(react_1.default.createElement(Checkbox_1.default, { data: false }));
            const el = container.querySelector('span');
            expect(el).toHaveTextContent('false');
        });
    });
});
