"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const deepmerge_1 = __importDefault(require("deepmerge"));
const sanitize_1 = __importDefault(require("../../fields/config/sanitize"));
const toKebabCase_1 = __importDefault(require("../../utilities/toKebabCase"));
const baseFields_1 = __importDefault(require("../../fields/baseFields/baseFields"));
const baseAPIKeyFields_1 = __importDefault(require("../../fields/baseFields/baseAPIKeyFields"));
const baseVerificationFields_1 = __importDefault(require("../../fields/baseFields/baseVerificationFields"));
const baseAccountLockFields_1 = __importDefault(require("../../fields/baseFields/baseAccountLockFields"));
const baseUploadFields_1 = __importDefault(require("../../fields/baseFields/baseUploadFields"));
const baseImageUploadFields_1 = __importDefault(require("../../fields/baseFields/baseImageUploadFields"));
const formatLabels_1 = __importDefault(require("../../utilities/formatLabels"));
const defaults_1 = require("./defaults");
const mergeBaseFields = (fields, baseFields) => {
    const mergedFields = [];
    if (fields) {
        baseFields.forEach((baseField) => {
            let matchedIndex = null;
            const match = fields.find((field, i) => {
                if (field.name === baseField.name) {
                    matchedIndex = i;
                    return true;
                }
                return false;
            });
            if (match) {
                const matchCopy = { ...match };
                fields.splice(matchedIndex, 1);
                let mergedField = {
                    ...baseField,
                    ...matchCopy,
                };
                if (baseField.fields && matchCopy.fields) {
                    mergedField.fields = mergeBaseFields(matchCopy.fields, baseField.fields);
                    return mergedFields.push(mergedField);
                }
                mergedField = deepmerge_1.default(mergedField, matchCopy, { arrayMerge: (_, source) => source });
                return mergedFields.push(mergedField);
            }
            return mergedFields.push(baseField);
        });
        return mergedFields;
    }
    return baseFields;
};
const sanitizeCollection = (collections, collection) => {
    // /////////////////////////////////
    // Make copy of collection config
    // /////////////////////////////////
    const sanitized = deepmerge_1.default(defaults_1.defaults, collection);
    sanitized.slug = toKebabCase_1.default(sanitized.slug);
    sanitized.labels = !sanitized.labels ? formatLabels_1.default(sanitized.slug) : sanitized.labels;
    if (sanitized.upload) {
        if (sanitized.upload === true)
            sanitized.upload = {};
        sanitized.upload.staticDir = sanitized.upload.staticDir || sanitized.slug;
        sanitized.upload.staticURL = sanitized.upload.staticURL || `/${sanitized.slug}`;
        sanitized.admin.useAsTitle = sanitized.admin.useAsTitle || 'filename';
        let uploadFields = baseUploadFields_1.default;
        if (sanitized.upload.imageSizes && Array.isArray(sanitized.upload.imageSizes)) {
            uploadFields = uploadFields.concat(baseImageUploadFields_1.default(sanitized.upload.imageSizes));
        }
        uploadFields = mergeBaseFields(sanitized.fields, uploadFields);
        sanitized.fields = [
            ...uploadFields,
            ...sanitized.fields,
        ];
    }
    if (sanitized.auth) {
        if (typeof collection.auth === 'object') {
            sanitized.auth = deepmerge_1.default(defaults_1.authDefaults, collection.auth);
        }
        else {
            sanitized.auth = {};
        }
        let authFields = baseFields_1.default;
        if (sanitized.auth.useAPIKey) {
            authFields = authFields.concat(baseAPIKeyFields_1.default);
        }
        if (sanitized.auth.verify) {
            if (sanitized.auth.verify === true)
                sanitized.auth.verify = {};
            authFields = authFields.concat(baseVerificationFields_1.default);
        }
        if (sanitized.auth.maxLoginAttempts > 0) {
            authFields = authFields.concat(baseAccountLockFields_1.default);
        }
        authFields = mergeBaseFields(sanitized.fields, authFields);
        sanitized.fields = [
            ...authFields,
            ...sanitized.fields,
        ];
    }
    // /////////////////////////////////
    // Sanitize fields
    // /////////////////////////////////
    const validRelationships = collections.map((c) => c.slug);
    sanitized.fields = sanitize_1.default(sanitized.fields, validRelationships);
    return sanitized;
};
exports.default = sanitizeCollection;
