"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const deepmerge_1 = __importDefault(require("deepmerge"));
const defaultUser_1 = __importDefault(require("../auth/defaultUser"));
const sanitize_1 = __importDefault(require("../collections/config/sanitize"));
const errors_1 = require("../errors");
const sanitize_2 = __importDefault(require("../globals/config/sanitize"));
const checkDuplicateCollections_1 = __importDefault(require("../utilities/checkDuplicateCollections"));
const defaults_1 = require("./defaults");
const sanitizeConfig = (config) => {
    const sanitizedConfig = deepmerge_1.default(defaults_1.defaults, config);
    sanitizedConfig.collections = sanitizedConfig.collections.map((collection) => sanitize_1.default(sanitizedConfig.collections, collection));
    checkDuplicateCollections_1.default(sanitizedConfig.collections);
    if (sanitizedConfig.globals.length > 0) {
        sanitizedConfig.globals = sanitize_2.default(sanitizedConfig.collections, sanitizedConfig.globals);
    }
    sanitizedConfig.csrf = [
        ...sanitizedConfig.csrf,
        config.serverURL,
    ];
    if (!sanitizedConfig.admin.user) {
        sanitizedConfig.admin.user = 'users';
        sanitizedConfig.collections.push(sanitize_1.default(sanitizedConfig.collections, defaultUser_1.default));
    }
    else if (!sanitizedConfig.collections.find((c) => c.slug === sanitizedConfig.admin.user)) {
        throw new errors_1.InvalidConfiguration(`${sanitizedConfig.admin.user} is not a valid admin user collection`);
    }
    return sanitizedConfig;
};
exports.default = sanitizeConfig;
