"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nodemailer_1 = __importDefault(require("nodemailer"));
const types_1 = require("../config/types");
const errors_1 = require("../errors");
const mockHandler_1 = __importDefault(require("./mockHandler"));
const logger_1 = __importDefault(require("../utilities/logger"));
const logger = logger_1.default();
async function handleTransport(transport, email) {
    try {
        await transport.verify();
    }
    catch (err) {
        logger.error('There is an error with the email configuration you have provided.', err);
    }
    return { ...email, transport };
}
const ensureConfigHasFrom = (emailConfig) => {
    if (!emailConfig.fromName || !emailConfig.fromAddress) {
        throw new errors_1.InvalidConfiguration('Email fromName and fromAddress must be configured when transport is configured');
    }
};
async function buildEmail(emailConfig) {
    if (types_1.hasTransport(emailConfig) && emailConfig.transport) {
        ensureConfigHasFrom(emailConfig);
        const email = { ...emailConfig };
        const { transport } = emailConfig;
        return handleTransport(transport, email);
    }
    if (types_1.hasTransportOptions(emailConfig) && emailConfig.transportOptions) {
        ensureConfigHasFrom(emailConfig);
        const email = { ...emailConfig };
        const transport = nodemailer_1.default.createTransport(emailConfig.transportOptions);
        return handleTransport(transport, email);
    }
    const mockAccount = await mockHandler_1.default(emailConfig);
    // Only log mock credentials if was explicitly set in config
    const { account: { web, user, pass } } = mockAccount;
    logger.info('E-mail configured with mock configuration');
    logger.info(`Log into mock email provider at ${web}`);
    logger.info(`Mock email account username: ${user}`);
    logger.info(`Mock email account password: ${pass}`);
    return mockAccount;
}
exports.default = buildEmail;
