"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const passport_1 = __importDefault(require("passport"));
const compression_1 = __importDefault(require("compression"));
const body_parser_1 = __importDefault(require("body-parser"));
const method_override_1 = __importDefault(require("method-override"));
const qs_middleware_1 = __importDefault(require("qs-middleware"));
const express_fileupload_1 = __importDefault(require("express-fileupload"));
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const middleware_1 = __importDefault(require("../../localization/middleware"));
const authenticate_1 = __importDefault(require("./authenticate"));
const identifyAPI_1 = __importDefault(require("./identifyAPI"));
const middleware = (payload) => {
    const rateLimitOptions = {
        windowMs: payload.config.rateLimit.window,
        max: payload.config.rateLimit.max,
    };
    if (typeof payload.config.rateLimit.skip === 'function')
        rateLimitOptions.skip = payload.config.rateLimit.skip;
    return [
        express_rate_limit_1.default(rateLimitOptions),
        passport_1.default.initialize(),
        identifyAPI_1.default('REST'),
        method_override_1.default('X-HTTP-Method-Override'),
        qs_middleware_1.default({ depth: 10 }),
        body_parser_1.default.urlencoded({ extended: true }),
        compression_1.default(payload.config.express.compression),
        middleware_1.default(payload.config.localization),
        express_1.default.json(payload.config.express.json),
        express_fileupload_1.default({
            parseNested: true,
            ...payload.config.upload,
        }),
        (req, res, next) => {
            if (payload.config.cors) {
                res.header('Access-Control-Allow-Methods', 'PUT, POST, GET, DELETE, OPTIONS');
                res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept, Authorization, Content-Encoding');
                if (payload.config.cors === '*') {
                    res.setHeader('Access-Control-Allow-Origin', '*');
                }
                else if (Array.isArray(payload.config.cors) && payload.config.cors.indexOf(req.headers.origin) > -1) {
                    res.header('Access-Control-Allow-Credentials', true);
                    res.setHeader('Access-Control-Allow-Origin', req.headers.origin);
                }
            }
            next();
        },
        authenticate_1.default(payload.config),
        ...(payload.config.express.middleware || []),
    ];
};
exports.default = middleware;
