"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../../errors");
const validations_1 = __importDefault(require("../validations"));
const sanitizeFields = (fields, validRelationships) => {
    if (!fields)
        return [];
    return fields.map((unsanitizedField) => {
        const field = { ...unsanitizedField };
        if (!field.type)
            throw new errors_1.MissingFieldType(field);
        if (field.type === 'relationship') {
            const relationships = Array.isArray(field.relationTo) ? field.relationTo : [field.relationTo];
            relationships.forEach((relationship) => {
                if (!validRelationships.includes(relationship)) {
                    throw new errors_1.InvalidFieldRelationship(field, relationship);
                }
            });
        }
        if (typeof field.validate === 'undefined') {
            const defaultValidate = validations_1.default[field.type];
            const noValidate = () => true;
            field.validate = defaultValidate || noValidate;
        }
        if (!field.hooks)
            field.hooks = {};
        if (!field.access)
            field.access = {};
        if (!field.admin)
            field.admin = {};
        if (field.fields)
            field.fields = sanitizeFields(field.fields, validRelationships);
        if (field.blocks) {
            field.blocks = field.blocks.map((block) => {
                const unsanitizedBlock = { ...block };
                unsanitizedBlock.fields = sanitizeFields(block.fields, validRelationships);
                return unsanitizedBlock;
            });
        }
        return field;
    });
};
exports.default = sanitizeFields;
