"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../../errors");
const sanitize_1 = __importDefault(require("../../fields/config/sanitize"));
const sanitizeGlobals = (collections, globals) => {
    // /////////////////////////////////
    // Ensure globals are valid
    // /////////////////////////////////
    globals.forEach((globalConfig) => {
        if (!globalConfig.label) {
            throw new errors_1.MissingGlobalLabel(globalConfig);
        }
    });
    const sanitizedGlobals = globals.map((global) => {
        const sanitizedGlobal = { ...global };
        // /////////////////////////////////
        // Ensure that collection has required object structure
        // /////////////////////////////////
        if (!sanitizedGlobal.hooks)
            sanitizedGlobal.hooks = {};
        if (!sanitizedGlobal.access)
            sanitizedGlobal.access = {};
        if (!sanitizedGlobal.admin)
            sanitizedGlobal.admin = {};
        if (!sanitizedGlobal.hooks.beforeValidate)
            sanitizedGlobal.hooks.beforeValidate = [];
        if (!sanitizedGlobal.hooks.beforeChange)
            sanitizedGlobal.hooks.beforeChange = [];
        if (!sanitizedGlobal.hooks.afterChange)
            sanitizedGlobal.hooks.afterChange = [];
        if (!sanitizedGlobal.hooks.beforeRead)
            sanitizedGlobal.hooks.beforeRead = [];
        if (!sanitizedGlobal.hooks.afterRead)
            sanitizedGlobal.hooks.afterRead = [];
        // /////////////////////////////////
        // Sanitize fields
        // /////////////////////////////////
        const validRelationships = collections.map((c) => c.slug);
        sanitizedGlobal.fields = sanitize_1.default(global.fields, validRelationships);
        return sanitizedGlobal;
    });
    return sanitizedGlobals;
};
exports.default = sanitizeGlobals;
