"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const executeAccess_1 = __importDefault(require("../../auth/executeAccess"));
const removeInternalFields_1 = __importDefault(require("../../utilities/removeInternalFields"));
async function findOne(args) {
    const { globals: { Model } } = this;
    const { globalConfig, req, slug, depth, showHiddenFields, } = args;
    // /////////////////////////////////////
    // 1. Retrieve and execute access
    // /////////////////////////////////////
    await executeAccess_1.default({ req }, globalConfig.access.read);
    // /////////////////////////////////////
    // 2. Perform database operation
    // /////////////////////////////////////
    let doc = await Model.findOne({ globalType: slug }).lean();
    if (!doc) {
        doc = {};
    }
    else if (doc._id) {
        doc.id = doc._id;
        delete doc._id;
    }
    // /////////////////////////////////////
    // 3. Execute before collection hook
    // /////////////////////////////////////
    await globalConfig.hooks.beforeRead.reduce(async (priorHook, hook) => {
        await priorHook;
        doc = await hook({
            req,
            doc,
        }) || doc;
    }, Promise.resolve());
    // /////////////////////////////////////
    // 4. Execute field-level hooks and access
    // /////////////////////////////////////
    doc = await this.performFieldOperations(globalConfig, {
        data: doc,
        hook: 'afterRead',
        operation: 'read',
        req,
        depth,
        reduceLocales: true,
        showHiddenFields,
    });
    // /////////////////////////////////////
    // 5. Execute after collection hook
    // /////////////////////////////////////
    await globalConfig.hooks.afterRead.reduce(async (priorHook, hook) => {
        await priorHook;
        doc = await hook({
            req,
            doc,
        }) || doc;
    }, Promise.resolve());
    // /////////////////////////////////////
    // 6. Return results
    // /////////////////////////////////////
    doc = removeInternalFields_1.default(doc);
    doc = JSON.stringify(doc);
    doc = JSON.parse(doc);
    return doc;
}
exports.default = findOne;
