"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const sharp_1 = __importDefault(require("sharp"));
const sanitize_filename_1 = __importDefault(require("sanitize-filename"));
const getImageSize_1 = __importDefault(require("./getImageSize"));
const fileExists_1 = __importDefault(require("./fileExists"));
function getOutputImage(sourceImage, size) {
    const extension = sourceImage.split('.').pop();
    const name = sanitize_filename_1.default(sourceImage.substr(0, sourceImage.lastIndexOf('.')) || sourceImage);
    return {
        name,
        extension,
        width: size.width,
        height: size.height,
    };
}
/**
 * @description
 * @param staticPath Path to save images
 * @param config Payload config
 * @param savedFilename
 * @param mimeType
 * @returns image sizes keyed to strings
 */
async function resizeAndSave(staticPath, config, savedFilename, mimeType) {
    const { imageSizes } = config.upload;
    const sourceImage = `${staticPath}/${savedFilename}`;
    const dimensions = await getImageSize_1.default(sourceImage);
    const sizes = imageSizes
        .filter((desiredSize) => desiredSize.width < dimensions.width)
        .map(async (desiredSize) => {
        const outputImage = getOutputImage(savedFilename, desiredSize);
        const imageNameWithDimensions = `${outputImage.name}-${outputImage.width}x${outputImage.height}.${outputImage.extension}`;
        const imagePath = `${staticPath}/${imageNameWithDimensions}`;
        const fileAlreadyExists = await fileExists_1.default(imagePath);
        if (fileAlreadyExists) {
            fs_1.default.unlinkSync(imagePath);
        }
        const output = await sharp_1.default(sourceImage)
            .resize(desiredSize.width, desiredSize.height, {
            position: desiredSize.crop || 'centre',
        })
            .toFile(imagePath);
        return {
            ...desiredSize,
            filename: imageNameWithDimensions,
            filesize: output.size,
            mimeType,
        };
    });
    const savedSizes = await Promise.all(sizes);
    return savedSizes.reduce((results, size) => ({
        ...results,
        [size.name]: {
            width: size.width,
            height: size.height,
            filename: size.filename,
            mimeType: size.mimeType,
            filesize: size.filesize,
        },
    }), {});
}
exports.default = resizeAndSave;
